/***************************************************************************
 *   Copyright (C) 2004 by JAVIER FERNANDO VARGAS GONZALEZ                 *
 *   JFVARG@linux                                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "pikdelay.h"
#include <math.h>

PiKDelay::PiKDelay(double ClkF, double TmDly)
{
    setClockFrequency(ClkF);
    setTimeDelay(TmDly);

    Refresh();
}


PiKDelay::~PiKDelay()
{
}


/*!
    \fn PiKDelay::Refresh()
 */
void PiKDelay::Refresh()
{
    CounterA = CounterB = CounterC = CounterD = -1;
    MachineCycleTime = 4.0/(ClockFrecuency*1000000.0);
    MachineCycles = floor(TimeDelay/MachineCycleTime);

    if(MachineCycles <= 766.0){
        CounterA = static_cast<int> ((MachineCycles-1.0)/3.0);
        MachineCycles = 1 + (3*CounterA);
    }
    if(MachineCycles > 766.0 && MachineCycles <= 196350.0){
        CounterB = static_cast<int>((MachineCycles + 765.0)/770.0);
        CounterA = static_cast<int>((MachineCycles + 765.0 -  (770.0 * CounterB))/3.0);
        MachineCycles = (3.0 * CounterA) + (770.0 * CounterB) - 765.0;
    }
    if(MachineCycles > 196350.0 && MachineCycles <= 50070524.0){
        CounterC = static_cast<int>((MachineCycles + 197116.0)/196355.0);
        CounterB = static_cast<int>((MachineCycles + 197116.0 - (196355.0 * CounterC))/770.0);
        CounterA = static_cast<int>((MachineCycles + 197116.0 - (196355.0 * CounterC) - (770.0 * CounterB))/3.0);
        MachineCycles = (3.0 * CounterA) + (770.0 * CounterB) + (196355.0 * CounterC) - 197116.0;
    }
    if(MachineCycles > 50070524.0 && MachineCycles <= 12767985148.0){
        CounterD = static_cast<int>((MachineCycles + 50267642.0)/50070530.0);
        CounterC = static_cast<int>((MachineCycles + 50267642.0 - (50070530.0 * CounterD))/196355.0);
        CounterB = static_cast<int>((MachineCycles + 50267642.0 - (50070530.0 * CounterD) - (196355.0 * CounterC))/770.0);
        CounterA = static_cast<int>((MachineCycles + 50267642.0 - (50070530.0 * CounterD) - (196355.0 * CounterC) - (CounterB * 770.0))/3.0);

        MachineCycles = (3.0 * CounterA) + (770.0 * CounterB) + (196355.0 * CounterC) + (50070530.0 * CounterD) - 50267642.0;
    }

    /*Waitting for correction*/
    if(MachineCycles > 12767985148.0){

    }

    TimeDelay = MachineCycles * MachineCycleTime;
}


/*!
    \fn PiKDelay::setClockFrecuency(double)
 */
void PiKDelay::setClockFrequency(double CF)
{
    if(CF > 0){
        ClockFrecuency = CF;
    }
    else{
        ClockFrecuency = 4.00;
    }
}


/*!
    \fn PiKDelay::setTimeDelay(double = 1.00)
 */
void PiKDelay::setTimeDelay(double TD)
{
    if(TD >= 0){
        TimeDelay = TD;
    }
    else{
        TimeDelay = 1.00;
    }
}


/*!
    \fn PiKDelay::getMachineCycles()
 */
double PiKDelay::getMachineCycles()
{
    Refresh();
    return MachineCycles;
}


/*!
    \fn PiKDelay::getTimeDelay()
 */
double PiKDelay::getTimeDelay()
{
    Refresh();
    return TimeDelay;
}


/*!
    \fn PiKDelay::getClockFrecuency()
 */
double PiKDelay::getClockFrecuency()
{
    Refresh();
    return ClockFrecuency;
}


/*!
    \fn PiKDelay::getCounterA()
 */
int PiKDelay::getCounterA()
{
    Refresh();
    return CounterA;
}


/*!
    \fn PiKDelay::getCounterB()
 */
int PiKDelay::getCounterB()
{
    Refresh();
    return CounterB;
}


/*!
    \fn PiKDelay::getCounterC()
 */
int PiKDelay::getCounterC()
{
    Refresh();
    return CounterC;
}


/*!
    \fn PiKDelay::getCounterD()
 */
int PiKDelay::getCounterD()
{
    Refresh();
    return CounterD;
}

